@@ -1,4 +1,3 @@
-/* $Id: android_jni_dev.c 4960 2014-11-13 01:48:34Z ming $ */
 /* 
  * Copyright (C) 2012-2012 Teluu Inc. (http://www.teluu.com)
  * Copyright (C) 2010-2012 Regis Montoya (aka r3gis - www.r3gis.fr)
@@ -37,8 +36,8 @@
 #include <sys/resource.h>
 #include <sys/system_properties.h>
 
-#define THIS_FILE	"android_jni_dev.c"
-#define DRIVER_NAME	"Android JNI"
+#define THIS_FILE       "android_jni_dev.c"
+#define DRIVER_NAME     "Android JNI"
 
 struct android_aud_factory
 {
@@ -59,7 +58,7 @@ struct android_aud_stream
     pj_str_t            name;
     pjmedia_dir         dir;
     pjmedia_aud_param   param;
-    
+
     int                 bytes_per_sample;
     pj_uint32_t         samples_per_sec;
     unsigned            samples_per_frame;
@@ -76,8 +75,8 @@ struct android_aud_stream
     pj_bool_t           rec_thread_exited;
     pj_thread_t        *rec_thread;
     pj_sem_t           *rec_sem;
-    pj_timestamp	rec_timestamp;
-    
+    pj_timestamp        rec_timestamp;    
+
     /* Track */
     jobject             track;
     jclass              track_class;
@@ -86,7 +85,7 @@ struct android_aud_stream
     pj_bool_t           play_thread_exited;
     pj_thread_t        *play_thread;
     pj_sem_t           *play_sem;
-    pj_timestamp	play_timestamp;
+    pj_timestamp        play_timestamp;
 };
 
 /* Factory prototypes */
@@ -141,81 +140,10 @@ static pjmedia_aud_stream_op android_strm_op =
     &strm_destroy
 };
 
-extern JavaVM *pj_jni_jvm;
-
-static pj_bool_t attach_jvm(JNIEnv **jni_env)
-{
-    if ((*pj_jni_jvm)->GetEnv(pj_jni_jvm, (void **)jni_env,
-                               JNI_VERSION_1_4) < 0)
-    {
-        if ((*pj_jni_jvm)->AttachCurrentThread(pj_jni_jvm, jni_env, NULL) < 0)
-        {
-            jni_env = NULL;
-            return PJ_FALSE;
-        }
-        return PJ_TRUE;
-    }
-    
-    return PJ_FALSE;
-}
-
-#define detach_jvm(attached) \
-    if (attached) \
-        (*pj_jni_jvm)->DetachCurrentThread(pj_jni_jvm);
-
-/* Thread priority utils */
-/* TODO : port it to pj_thread functions */
-#define THREAD_PRIORITY_AUDIO -16
-#define THREAD_PRIORITY_URGENT_AUDIO -19
-
-pj_status_t set_android_thread_priority(int priority)
-{
-    jclass process_class;
-    jmethodID set_prio_method;
-    jthrowable exc;
-    pj_status_t result = PJ_SUCCESS;
-    JNIEnv *jni_env = 0;
-    pj_bool_t attached = attach_jvm(&jni_env);
-    
-    PJ_ASSERT_RETURN(jni_env, PJ_FALSE);
-
-    /* Get pointer to the java class */
-    process_class = (jclass)(*jni_env)->NewGlobalRef(jni_env, 
-                        (*jni_env)->FindClass(jni_env, "android/os/Process"));
-    if (process_class == 0) {
-        PJ_LOG(4, (THIS_FILE, "Unable to find os process class"));
-        result = PJ_EIGNORED;
-        goto on_return;
-    }
-    
-    /* Get the id of set thread priority function */
-    set_prio_method = (*jni_env)->GetStaticMethodID(jni_env, process_class,
-                                                    "setThreadPriority",
-                                                    "(I)V");
-    if (set_prio_method == 0) {
-        PJ_LOG(4, (THIS_FILE, "Unable to find setThreadPriority() method"));
-        result = PJ_EIGNORED;
-        goto on_return;
-    }
-    
-    /* Set the thread priority */
-    (*jni_env)->CallStaticVoidMethod(jni_env, process_class, set_prio_method,
-                                     priority);    
-    exc = (*jni_env)->ExceptionOccurred(jni_env);
-    if (exc) {
-        (*jni_env)->ExceptionDescribe(jni_env);
-        (*jni_env)->ExceptionClear(jni_env);
-        PJ_LOG(4, (THIS_FILE, "Failure in setting thread priority using "
-                              "Java API, fallback to setpriority()"));
-        setpriority(PRIO_PROCESS, 0, priority);
-    } else {
-        PJ_LOG(4, (THIS_FILE, "Setting thread priority successful"));        
-    }
-
-on_return:
-    detach_jvm(attached);
-    return result;
-}
+#define attach_jvm(jni_env)     pj_jni_attach_jvm((void **)jni_env)
+#define detach_jvm(attached)    pj_jni_detach_jvm(attached)
+#define THREAD_PRIORITY_AUDIO           -16
+#define THREAD_PRIORITY_URGENT_AUDIO    -19
 
 
 static int AndroidRecorderCallback(void *userData)
@@ -255,15 +183,8 @@ static int AndroidRecorderCallback(void *userData)
         goto on_return;
     }
     
-    /* Start recording
-     * setpriority(PRIO_PROCESS, 0, -19); //ANDROID_PRIORITY_AUDIO
-     * set priority is probably not enough because it does not change the thread
-     * group in scheduler
-     * Temporary solution is to call the java api to set the thread priority.
-     * A cool solution would be to port (if possible) the code from the
-     * android os regarding set_sched groups
-     */
-    set_android_thread_priority(THREAD_PRIORITY_URGENT_AUDIO);
+    /* Start recording */
+    pj_thread_set_prio(NULL, THREAD_PRIORITY_URGENT_AUDIO);
     (*jni_env)->CallVoidMethod(jni_env, stream->record, record_method);
     
     while (!stream->quit_flag) {
@@ -297,7 +218,9 @@ static int AndroidRecorderCallback(void *userData)
 
         status = (*stream->rec_cb)(stream->user_data, &frame);
         (*jni_env)->ReleaseByteArrayElements(jni_env, inputBuffer, buf,
-        				     JNI_ABORT);
+                                             JNI_ABORT);
+        if (status != PJ_SUCCESS || stream->quit_flag)
+            break;
 
         stream->rec_timestamp.u64 += stream->param.samples_per_frame /
                                      stream->param.channel_count;
@@ -354,7 +277,7 @@ static int AndroidTrackCallback(void *userData)
     buf = (*jni_env)->GetByteArrayElements(jni_env, outputBuffer, 0);
 
     /* Start playing */
-    set_android_thread_priority(THREAD_PRIORITY_URGENT_AUDIO);
+    pj_thread_set_prio(NULL, THREAD_PRIORITY_URGENT_AUDIO);
     (*jni_env)->CallVoidMethod(jni_env, stream->track, play_method);
 
     while (!stream->quit_flag) {
@@ -385,7 +308,7 @@ static int AndroidTrackCallback(void *userData)
             pj_bzero(frame.buf, frame.size);
         
         (*jni_env)->ReleaseByteArrayElements(jni_env, outputBuffer, buf,
-        				     JNI_COMMIT);
+                                             JNI_COMMIT);
 
         /* Write to the device output. */
         bytesWritten = (*jni_env)->CallIntMethod(jni_env, stream->track,
@@ -479,11 +402,14 @@ static pj_status_t android_get_dev_info(pjmedia_aud_dev_factory *f,
     
     pj_bzero(info, sizeof(*info));
     
-    pj_ansi_strcpy(info->name, "Android JNI");
+    pj_ansi_strxcpy(info->name, "Android JNI", sizeof(info->name));
+    pj_ansi_strxcpy(info->driver, DRIVER_NAME, sizeof(info->driver));
     info->default_samples_per_sec = 8000;
-    info->caps = PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING;
+    info->caps = PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING |
+                 PJMEDIA_AUD_DEV_CAP_INPUT_SOURCE;
     info->input_count = 1;
     info->output_count = 1;
+    info->routes = PJMEDIA_AUD_DEV_ROUTE_CUSTOM;
     
     return PJ_SUCCESS;
 }
@@ -498,7 +424,7 @@ static pj_status_t android_default_param(pjmedia_aud_dev_factory *f,
     
     status = android_get_dev_info(f, index, &adi);
     if (status != PJ_SUCCESS)
-	return status;
+        return status;
     
     pj_bzero(param, sizeof(*param));
     if (adi.input_count && adi.output_count) {
@@ -540,7 +466,7 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
     struct android_aud_stream *stream;
     pj_status_t status = PJ_SUCCESS;
     int state = 0;
-    int buffSize, inputBuffSizePlay, inputBuffSizeRec;
+    int buffSize, inputBuffSizePlay = 0, inputBuffSizeRec = 0;
     int channelInCfg, channelOutCfg, sampleFormat;
     jmethodID constructor_method=0, bufsize_method = 0;
     jmethodID method_id = 0;
@@ -563,7 +489,7 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
     stream = PJ_POOL_ZALLOC_T(pool, struct android_aud_stream);
     stream->pool = pool;
     pj_strdup2_with_null(pool, &stream->name, "JNI stream");
-    stream->dir = PJMEDIA_DIR_CAPTURE_PLAYBACK;
+    stream->dir = param->dir;
     pj_memcpy(&stream->param, param, sizeof(*param));
     stream->user_data = user_data;
     stream->rec_cb = rec_cb;
@@ -661,8 +587,15 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
     
     if (stream->dir & PJMEDIA_DIR_CAPTURE) {
         jthrowable exc;
+        jobject record_obj;
         int mic_source = 0; /* DEFAULT: default audio source */
 
+        if ((param->flags & PJMEDIA_AUD_DEV_CAP_INPUT_SOURCE) &&
+            (param->input_route & PJMEDIA_AUD_DEV_ROUTE_CUSTOM))
+        {
+            mic_source = param->input_route & ~PJMEDIA_AUD_DEV_ROUTE_CUSTOM;
+        }
+
         /* Get pointer to the constructor */
         constructor_method = (*jni_env)->GetMethodID(jni_env,
                                                      stream->record_class,
@@ -676,9 +609,9 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
         if (mic_source == 0) {
             /* Android-L (android-21) removes __system_property_get
              * from the NDK.
-	     */
-	    /*           
-	    char sdk_version[PROP_VALUE_MAX];
+             */
+            /*           
+            char sdk_version[PROP_VALUE_MAX];
             pj_str_t pj_sdk_version;
             int sdk_v;
 
@@ -692,15 +625,15 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
         PJ_LOG(4, (THIS_FILE, "Using audio input source : %d", mic_source));
         
         do {
-            stream->record =  (*jni_env)->NewObject(jni_env,
-                                                    stream->record_class,
-                                                    constructor_method,
-                                                    mic_source, 
-                                                    param->clock_rate,
-                                                    channelInCfg,
-                                                    sampleFormat,
-                                                    inputBuffSizeRec);
-            if (stream->record == 0) {
+            record_obj =  (*jni_env)->NewObject(jni_env,
+                                                stream->record_class,
+                                                constructor_method,
+                                                mic_source, 
+                                                param->clock_rate,
+                                                channelInCfg,
+                                                sampleFormat,
+                                                inputBuffSizeRec);
+            if (record_obj == 0) {
                 PJ_LOG(3, (THIS_FILE, "Unable to create audio record object"));
                 status = PJMEDIA_EAUD_INIT;
                 goto on_error;
@@ -729,8 +662,7 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
                 status = PJMEDIA_EAUD_SYSERR;
                 goto on_error;
             }
-            state = (*jni_env)->CallIntMethod(jni_env, stream->record,
-                                              method_id);
+            state = (*jni_env)->CallIntMethod(jni_env, record_obj, method_id);
             if (state == 0) { /* STATE_UNINITIALIZED */
                 PJ_LOG(3, (THIS_FILE, "Failure in initializing audio record."));
                 if (mic_source == 0) {
@@ -742,9 +674,16 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
             }
         } while (state == 0);
         
-        stream->record = (*jni_env)->NewGlobalRef(jni_env, stream->record);
+        stream->record = (*jni_env)->NewGlobalRef(jni_env, record_obj);
         if (stream->record == 0) {
-            PJ_LOG(3, (THIS_FILE, "Unable to create audio record global ref."));
+            jmethodID release_method=0;
+            
+            PJ_LOG(3, (THIS_FILE, "Unable to create audio record global ref."));            
+            release_method = (*jni_env)->GetMethodID(jni_env, 
+                                                     stream->record_class,
+                                                     "release", "()V");
+            (*jni_env)->CallVoidMethod(jni_env, record_obj, release_method);
+            
             status = PJMEDIA_EAUD_INIT;
             goto on_error;
         }
@@ -764,6 +703,7 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
     
     if (stream->dir & PJMEDIA_DIR_PLAYBACK) {
         jthrowable exc;
+        jobject track_obj;
         
         /* Get pointer to the constructor */
         constructor_method = (*jni_env)->GetMethodID(jni_env,
@@ -775,16 +715,16 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
             goto on_error;
         }
         
-        stream->track = (*jni_env)->NewObject(jni_env,
-                                              stream->track_class,
-                                              constructor_method,
-                                              0, /* STREAM_VOICE_CALL */
-                                              param->clock_rate,
-                                              channelOutCfg,
-                                              sampleFormat,
-                                              inputBuffSizePlay,
-                                              1 /* MODE_STREAM */);
-        if (stream->track == 0) {
+        track_obj = (*jni_env)->NewObject(jni_env,
+                                          stream->track_class,
+                                          constructor_method,
+                                          0, /* STREAM_VOICE_CALL */
+                                          param->clock_rate,
+                                          channelOutCfg,
+                                          sampleFormat,
+                                          inputBuffSizePlay,
+                                          1 /* MODE_STREAM */);
+        if (track_obj == 0) {
             PJ_LOG(3, (THIS_FILE, "Unable to create audio track object."));
             status = PJMEDIA_EAUD_INIT;
             goto on_error;
@@ -799,8 +739,15 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
             goto on_error;
         }
         
-        stream->track = (*jni_env)->NewGlobalRef(jni_env, stream->track);
+        stream->track = (*jni_env)->NewGlobalRef(jni_env, track_obj);
         if (stream->track == 0) {
+            jmethodID release_method=0;
+                
+            release_method = (*jni_env)->GetMethodID(jni_env, 
+                                                     stream->track_class,
+                                                     "release", "()V");
+            (*jni_env)->CallVoidMethod(jni_env, track_obj, release_method);
+            
             PJ_LOG(3, (THIS_FILE, "Unable to create audio track's global ref"));
             status = PJMEDIA_EAUD_INIT;
             goto on_error;
@@ -837,7 +784,7 @@ static pj_status_t android_create_stream(pjmedia_aud_dev_factory *f,
     }
 
     if (param->flags & PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING) {
-	strm_set_cap(&stream->base, PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING,
+        strm_set_cap(&stream->base, PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING,
                      &param->output_vol);
     }
     
@@ -883,7 +830,7 @@ static pj_status_t strm_get_cap(pjmedia_aud_stream *s,
     PJ_ASSERT_RETURN(s && pval, PJ_EINVAL);
     
     if (cap==PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING &&
-	(strm->param.dir & PJMEDIA_DIR_PLAYBACK))
+        (strm->param.dir & PJMEDIA_DIR_PLAYBACK))
     {
     }
     
@@ -902,11 +849,11 @@ static pj_status_t strm_set_cap(pjmedia_aud_stream *s,
     PJ_ASSERT_RETURN(s && value, PJ_EINVAL);
     
     if (cap==PJMEDIA_AUD_DEV_CAP_OUTPUT_VOLUME_SETTING &&
-	(stream->param.dir & PJMEDIA_DIR_PLAYBACK))
+        (stream->param.dir & PJMEDIA_DIR_PLAYBACK))
     {
         if (stream->track) {
             jmethodID vol_method = 0;
-            int retval;
+            int retval = 0;
             float vol = *(int *)value;
             
             attached = attach_jvm(&jni_env);
@@ -970,14 +917,14 @@ static pj_status_t strm_destroy(pjmedia_aud_stream *s)
     pj_bool_t attached;
     
     PJ_LOG(4,(THIS_FILE, "Destroying Android JNI stream..."));
-    
+
     stream->quit_flag = PJ_TRUE;
     
     /* Stop the stream */
     strm_stop(s);
     
     attached = attach_jvm(&jni_env);
-    
+
     if (stream->record){
         if (stream->rec_thread) {
             pj_sem_post(stream->rec_sem);
@@ -990,17 +937,21 @@ static pj_status_t strm_destroy(pjmedia_aud_stream *s)
             pj_sem_destroy(stream->rec_sem);
             stream->rec_sem = NULL;
         }
-        
-        release_method = (*jni_env)->GetMethodID(jni_env, stream->record_class,
-                                                 "release", "()V");
-        (*jni_env)->CallVoidMethod(jni_env, stream->record, release_method);
-        
+        if (stream->record_class) {
+            release_method = (*jni_env)->GetMethodID(jni_env, 
+                                                     stream->record_class,
+                                                     "release", "()V");
+            (*jni_env)->CallVoidMethod(jni_env, stream->record,
+                                       release_method);
+        }
         (*jni_env)->DeleteGlobalRef(jni_env, stream->record);
-        (*jni_env)->DeleteGlobalRef(jni_env, stream->record_class);
         stream->record = NULL;
-        stream->record_class = NULL;
         PJ_LOG(4, (THIS_FILE, "Audio record released"));
     }
+    if (stream->record_class) {
+        (*jni_env)->DeleteGlobalRef(jni_env, stream->record_class);
+        stream->record_class = NULL;
+    }
     
     if (stream->track) {
         if (stream->play_thread) {
@@ -1014,16 +965,20 @@ static pj_status_t strm_destroy(pjmedia_aud_stream *s)
             pj_sem_destroy(stream->play_sem);
             stream->play_sem = NULL;
         }
-        
-        release_method = (*jni_env)->GetMethodID(jni_env, stream->track_class,
-                                                 "release", "()V");
-        (*jni_env)->CallVoidMethod(jni_env, stream->track, release_method);
-        
+        if (stream->track_class) {
+            release_method = (*jni_env)->GetMethodID(jni_env, 
+                                                     stream->track_class,
+                                                     "release", "()V");
+            (*jni_env)->CallVoidMethod(jni_env, stream->track, 
+                                       release_method);
+        }
         (*jni_env)->DeleteGlobalRef(jni_env, stream->track);
+        stream->track = NULL;        
+        PJ_LOG(4, (THIS_FILE, "Audio track released"));
+    }
+    if (stream->track_class) {
         (*jni_env)->DeleteGlobalRef(jni_env, stream->track_class);
-        stream->track = NULL;
         stream->track_class = NULL;
-        PJ_LOG(3, (THIS_FILE, "Audio track released"));
     }
 
     pj_pool_release(stream->pool);
@@ -1033,4 +988,4 @@ static pj_status_t strm_destroy(pjmedia_aud_stream *s)
     return PJ_SUCCESS;
 }
 
-#endif	/* PJMEDIA_AUDIO_DEV_HAS_ANDROID_JNI */
+#endif  /* PJMEDIA_AUDIO_DEV_HAS_ANDROID_JNI */
