@@ -1,4 +1,3 @@
-/* $Id: silk.c 5046 2015-04-06 06:21:41Z nanang $ */
 /* 
  * Copyright (C) 2012-2012 Teluu Inc. (http://www.teluu.com)
  * Contributed by Regis Montoya (aka r3gis - www.r3gis.fr)
@@ -33,7 +32,7 @@
 
 #include "SKP_Silk_SDK_API.h"
 
-#define THIS_FILE		"silk.c"
+#define THIS_FILE               "silk.c"
 
 #ifndef PJMEDIA_SILK_DELAY_BUF_OPTIONS
     #define PJMEDIA_SILK_DELAY_BUF_OPTIONS PJMEDIA_DELAY_BUF_SIMPLE_FIFO
@@ -51,65 +50,65 @@
 
 /* Prototypes for SILK factory */
 static pj_status_t silk_test_alloc( pjmedia_codec_factory *factory,
-				    const pjmedia_codec_info *id );
+                                    const pjmedia_codec_info *id );
 static pj_status_t silk_default_attr( pjmedia_codec_factory *factory,
-				      const pjmedia_codec_info *id,
-				      pjmedia_codec_param *attr );
+                                      const pjmedia_codec_info *id,
+                                      pjmedia_codec_param *attr );
 static pj_status_t silk_enum_codecs ( pjmedia_codec_factory *factory,
-				      unsigned *count,
-				      pjmedia_codec_info codecs[]);
+                                      unsigned *count,
+                                      pjmedia_codec_info codecs[]);
 static pj_status_t silk_alloc_codec( pjmedia_codec_factory *factory,
-				     const pjmedia_codec_info *id,
-				     pjmedia_codec **p_codec);
+                                     const pjmedia_codec_info *id,
+                                     pjmedia_codec **p_codec);
 static pj_status_t silk_dealloc_codec( pjmedia_codec_factory *factory,
-				       pjmedia_codec *codec );
+                                       pjmedia_codec *codec );
 
 /* Prototypes for SILK implementation. */
 static pj_status_t  silk_codec_init( pjmedia_codec *codec,
-				     pj_pool_t *pool );
+                                     pj_pool_t *pool );
 static pj_status_t  silk_codec_open( pjmedia_codec *codec,
-				     pjmedia_codec_param *attr );
+                                     pjmedia_codec_param *attr );
 static pj_status_t  silk_codec_close( pjmedia_codec *codec );
 static pj_status_t  silk_codec_modify( pjmedia_codec *codec,
-				       const pjmedia_codec_param *attr );
+                                       const pjmedia_codec_param *attr );
 static pj_status_t  silk_codec_parse( pjmedia_codec *codec,
-				      void *pkt,
-				      pj_size_t pkt_size,
-				      const pj_timestamp *timestamp,
-				      unsigned *frame_cnt,
-				      pjmedia_frame frames[]);
+                                      void *pkt,
+                                      pj_size_t pkt_size,
+                                      const pj_timestamp *timestamp,
+                                      unsigned *frame_cnt,
+                                      pjmedia_frame frames[]);
 static pj_status_t  silk_codec_encode( pjmedia_codec *codec,
-				       const struct pjmedia_frame *input,
-				       unsigned output_buf_len,
-				       struct pjmedia_frame *output);
+                                       const struct pjmedia_frame *input,
+                                       unsigned output_buf_len,
+                                       struct pjmedia_frame *output);
 static pj_status_t  silk_codec_decode( pjmedia_codec *codec,
-				       const struct pjmedia_frame *input,
-				       unsigned output_buf_len,
-				       struct pjmedia_frame *output);
+                                       const struct pjmedia_frame *input,
+                                       unsigned output_buf_len,
+                                       struct pjmedia_frame *output);
 static pj_status_t  silk_codec_recover( pjmedia_codec *codec,
-					unsigned output_buf_len,
-					struct pjmedia_frame *output);
+                                        unsigned output_buf_len,
+                                        struct pjmedia_frame *output);
 
 
 typedef enum
 {
-    PARAM_NB,   /* Index for narrowband parameter.	*/
-    PARAM_MB,	/* Index for medium parameter.		*/
-    PARAM_WB,	/* Index for wideband parameter.	*/
-    PARAM_SWB,	/* Index for super-wideband parameter	*/
+    PARAM_NB,   /* Index for narrowband parameter.      */
+    PARAM_MB,   /* Index for medium parameter.          */
+    PARAM_WB,   /* Index for wideband parameter.        */
+    PARAM_SWB,  /* Index for super-wideband parameter   */
 } silk_mode;
 
 
 /* Silk default parameter */
 typedef struct silk_param
 {
-    int		 enabled;	    /* Is this mode enabled?		    */
-    int		 pt;		    /* Payload type.			    */
-    unsigned	 clock_rate;	    /* Default sampling rate to be used.    */
-    pj_uint16_t	 ptime;		    /* packet length (in ms).		    */
-    pj_uint32_t  bitrate;	    /* Bit rate for current mode.	    */
-    pj_uint32_t  max_bitrate;	    /* Max bit rate for current mode.	    */
-    int 	 complexity;	    /* Complexity mode: 0/lowest to 2.	    */
+    int          enabled;           /* Is this mode enabled?                */
+    int          pt;                /* Payload type.                        */
+    unsigned     clock_rate;        /* Default sampling rate to be used.    */
+    pj_uint16_t  ptime;             /* packet length (in ms).               */
+    pj_uint32_t  bitrate;           /* Bit rate for current mode.           */
+    pj_uint32_t  max_bitrate;       /* Max bit rate for current mode.       */
+    int          complexity;        /* Complexity mode: 0/lowest to 2.      */
 } silk_param;
 
 
@@ -141,29 +140,29 @@ static pjmedia_codec_factory_op silk_factory_op =
 /* SILK factory private data */
 static struct silk_factory
 {
-    pjmedia_codec_factory	base;
-    pjmedia_endpt	       *endpt;
-    pj_pool_t		       *pool;
-    pj_mutex_t		       *mutex;
-    struct silk_param		silk_param[4];
+    pjmedia_codec_factory       base;
+    pjmedia_endpt              *endpt;
+    pj_pool_t                  *pool;
+    pj_mutex_t                 *mutex;
+    struct silk_param           silk_param[4];
 } silk_factory;
 
 
 /* SILK codec private data. */
 typedef struct silk_private
 {
-    silk_mode	 mode;		/**< Silk mode.	*/
-    pj_pool_t	*pool;		/**< Pool for each instance.    */
-    unsigned	 samples_per_frame;
+    silk_mode    mode;          /**< Silk mode. */
+    pj_pool_t   *pool;          /**< Pool for each instance.    */
+    unsigned     samples_per_frame;
     pj_uint8_t   pcm_bytes_per_sample;
 
-    pj_bool_t	 enc_ready;
+    pj_bool_t    enc_ready;
     SKP_SILK_SDK_EncControlStruct enc_ctl;
-    void	*enc_st;
+    void        *enc_st;
 
-    pj_bool_t	 dec_ready;
+    pj_bool_t    dec_ready;
     SKP_SILK_SDK_DecControlStruct dec_ctl;
-    void	*dec_st;
+    void        *dec_st;
 
     /* Buffer to hold decoded frames. */
     void        *dec_buf[SILK_MAX_FRAMES_PER_PACKET-1];
@@ -176,11 +175,11 @@ typedef struct silk_private
 
 silk_mode silk_get_mode_from_clock_rate(unsigned clock_rate) {
     if (clock_rate <= silk_factory.silk_param[PARAM_NB].clock_rate) {
-	return PARAM_NB;
+        return PARAM_NB;
     } else if (clock_rate <= silk_factory.silk_param[PARAM_MB].clock_rate) {
-	return PARAM_MB;
+        return PARAM_MB;
     } else if (clock_rate <= silk_factory.silk_param[PARAM_WB].clock_rate) {
-	return PARAM_WB;
+        return PARAM_WB;
     }
     return PARAM_SWB;
 }
@@ -193,8 +192,8 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_init(pjmedia_endpt *endpt)
     pj_status_t status;
 
     if (silk_factory.endpt != NULL) {
-	/* Already initialized. */
-	return PJ_SUCCESS;
+        /* Already initialized. */
+        return PJ_SUCCESS;
     }
 
     /* Init factory */
@@ -206,13 +205,13 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_init(pjmedia_endpt *endpt)
     /* Create pool */
     silk_factory.pool = pjmedia_endpt_create_pool(endpt, "silk", 4000, 4000);
     if (!silk_factory.pool)
-	return PJ_ENOMEM;
+        return PJ_ENOMEM;
 
     /* Create mutex. */
     status = pj_mutex_create_simple(silk_factory.pool, "silk",
-				    &silk_factory.mutex);
+                                    &silk_factory.mutex);
     if (status != PJ_SUCCESS)
-	goto on_error;
+        goto on_error;
 
     /* Initialize default codec params */
 
@@ -270,27 +269,27 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_init(pjmedia_endpt *endpt)
     /* Get the codec manager. */
     codec_mgr = pjmedia_endpt_get_codec_mgr(endpt);
     if (!codec_mgr) {
-	return PJ_EINVALIDOP;
+        return PJ_EINVALIDOP;
     }
 
     /* Register codec factory to endpoint. */
     status = pjmedia_codec_mgr_register_factory(codec_mgr,
-						&silk_factory.base);
+                                                &silk_factory.base);
     if (status != PJ_SUCCESS)
-	return status;
+        return status;
 
     PJ_LOG(4,(THIS_FILE, "SILK codec version %s initialized",
-	      SKP_Silk_SDK_get_version()));
+              SKP_Silk_SDK_get_version()));
     return PJ_SUCCESS;
 
 on_error:
     if (silk_factory.mutex) {
-	pj_mutex_destroy(silk_factory.mutex);
-	silk_factory.mutex = NULL;
+        pj_mutex_destroy(silk_factory.mutex);
+        silk_factory.mutex = NULL;
     }
     if (silk_factory.pool) {
-	pj_pool_release(silk_factory.pool);
-	silk_factory.pool = NULL;
+        pj_pool_release(silk_factory.pool);
+        silk_factory.pool = NULL;
     }
 
     return status;
@@ -302,20 +301,19 @@ on_error:
  * clock rate.
  */
 PJ_DEF(pj_status_t) pjmedia_codec_silk_set_config(
-				    unsigned clock_rate, 
-				    const pjmedia_codec_silk_setting *opt)
+                                    unsigned clock_rate, 
+                                    const pjmedia_codec_silk_setting *opt)
 {
     unsigned i;
 
     /* Look up in factory modes table */
-    for (i = 0; i < sizeof(silk_factory.silk_param)/
-                    sizeof(silk_factory.silk_param[0]); ++i)
+    for (i = 0; i < PJ_ARRAY_SIZE(silk_factory.silk_param); ++i)
     {
         if (silk_factory.silk_param[i].clock_rate == clock_rate) {
             int quality = PJMEDIA_CODEC_SILK_DEFAULT_QUALITY;
             int complexity = PJMEDIA_CODEC_SILK_DEFAULT_COMPLEXITY;
 
-	    silk_factory.silk_param[i].enabled = opt->enabled;
+            silk_factory.silk_param[i].enabled = opt->enabled;
             if (opt->complexity >= 0)
                 complexity = opt->complexity;
             silk_factory.silk_param[i].complexity = complexity;
@@ -327,8 +325,8 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_set_config(
             if (silk_factory.silk_param[i].bitrate < SILK_MIN_BITRATE)
                 silk_factory.silk_param[i].bitrate = SILK_MIN_BITRATE;
 
-	    return PJ_SUCCESS;
-	}
+            return PJ_SUCCESS;
+        }
     }
 
     return PJ_ENOTFOUND;
@@ -345,8 +343,8 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_deinit(void)
     pj_status_t status;
 
     if (silk_factory.endpt == NULL) {
-	/* Not registered. */
-	return PJ_SUCCESS;
+        /* Not registered. */
+        return PJ_SUCCESS;
     }
 
     /* Lock mutex. */
@@ -355,14 +353,14 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_deinit(void)
     /* Get the codec manager. */
     codec_mgr = pjmedia_endpt_get_codec_mgr(silk_factory.endpt);
     if (!codec_mgr) {
-	silk_factory.endpt = NULL;
-	pj_mutex_unlock(silk_factory.mutex);
-	return PJ_EINVALIDOP;
+        silk_factory.endpt = NULL;
+        pj_mutex_unlock(silk_factory.mutex);
+        return PJ_EINVALIDOP;
     }
 
     /* Unregister silk codec factory. */
     status = pjmedia_codec_mgr_unregister_factory(codec_mgr,
-						  &silk_factory.base);
+                                                  &silk_factory.base);
     silk_factory.endpt = NULL;
 
     /* Destroy mutex. */
@@ -383,7 +381,7 @@ PJ_DEF(pj_status_t) pjmedia_codec_silk_deinit(void)
  * Check if factory can allocate the specified codec.
  */
 static pj_status_t silk_test_alloc(pjmedia_codec_factory *factory,
-				   const pjmedia_codec_info *info )
+                                   const pjmedia_codec_info *info )
 {
     const pj_str_t silk_tag = {"SILK", 4};
     unsigned i;
@@ -393,23 +391,23 @@ static pj_status_t silk_test_alloc(pjmedia_codec_factory *factory,
 
     /* Type MUST be audio. */
     if (info->type != PJMEDIA_TYPE_AUDIO)
-	return PJMEDIA_CODEC_EUNSUP;
+        return PJMEDIA_CODEC_EUNSUP;
 
     /* Check encoding name. */
     if (pj_stricmp(&info->encoding_name, &silk_tag) != 0)
-	return PJMEDIA_CODEC_EUNSUP;
+        return PJMEDIA_CODEC_EUNSUP;
 
     /* Channel count must be one */
     if (info->channel_cnt != 1)
-	return PJMEDIA_CODEC_EUNSUP;
+        return PJMEDIA_CODEC_EUNSUP;
 
     /* Check clock-rate */
     for (i=0; i<PJ_ARRAY_SIZE(silk_factory.silk_param); ++i) {
-	silk_param *sp = &silk_factory.silk_param[i];
-	if (sp->enabled && info->clock_rate == sp->clock_rate)
-	{
-	    return PJ_SUCCESS;
-	}
+        silk_param *sp = &silk_factory.silk_param[i];
+        if (sp->enabled && info->clock_rate == sp->clock_rate)
+        {
+            return PJ_SUCCESS;
+        }
     }
     /* Clock rate not supported */
     return PJMEDIA_CODEC_EUNSUP;
@@ -420,8 +418,8 @@ static pj_status_t silk_test_alloc(pjmedia_codec_factory *factory,
  * Generate default attribute.
  */
 static pj_status_t silk_default_attr( pjmedia_codec_factory *factory,
-				      const pjmedia_codec_info *id,
-				      pjmedia_codec_param *attr )
+                                      const pjmedia_codec_info *id,
+                                      pjmedia_codec_param *attr )
 {
     silk_param *sp;
     int i;
@@ -462,8 +460,8 @@ static pj_status_t silk_default_attr( pjmedia_codec_factory *factory,
  * Enum codecs supported by this factory.
  */
 static pj_status_t silk_enum_codecs(pjmedia_codec_factory *factory,
-				    unsigned *count,
-				    pjmedia_codec_info codecs[])
+                                    unsigned *count,
+                                    pjmedia_codec_info codecs[])
 {
     unsigned max;
     int i;
@@ -474,21 +472,22 @@ static pj_status_t silk_enum_codecs(pjmedia_codec_factory *factory,
     max = *count;
     *count = 0;
 
-    for (i = 0; i<PJ_ARRAY_SIZE(silk_factory.silk_param) && *count<max; ++i)
+    for (i = 0; i<(int)PJ_ARRAY_SIZE(silk_factory.silk_param) && *count<max;
+         ++i)
     {
-	silk_param *sp = &silk_factory.silk_param[i];
+        silk_param *sp = &silk_factory.silk_param[i];
 
-    	if (!sp->enabled)
-    	    continue;
+        if (!sp->enabled)
+            continue;
 
-    	pj_bzero(&codecs[*count], sizeof(pjmedia_codec_info));
-    	codecs[*count].encoding_name = pj_str("SILK");
-    	codecs[*count].pt = sp->pt;
-    	codecs[*count].type = PJMEDIA_TYPE_AUDIO;
-    	codecs[*count].clock_rate = sp->clock_rate;
-    	codecs[*count].channel_cnt = 1;
+        pj_bzero(&codecs[*count], sizeof(pjmedia_codec_info));
+        codecs[*count].encoding_name = pj_str("SILK");
+        codecs[*count].pt = sp->pt;
+        codecs[*count].type = PJMEDIA_TYPE_AUDIO;
+        codecs[*count].clock_rate = sp->clock_rate;
+        codecs[*count].channel_cnt = 1;
 
-    	++*count;
+        ++*count;
     }
 
     return PJ_SUCCESS;
@@ -499,8 +498,8 @@ static pj_status_t silk_enum_codecs(pjmedia_codec_factory *factory,
  * Allocate a new SILK codec instance.
  */
 static pj_status_t silk_alloc_codec(pjmedia_codec_factory *factory,
-				    const pjmedia_codec_info *id,
-				    pjmedia_codec **p_codec)
+                                    const pjmedia_codec_info *id,
+                                    pjmedia_codec **p_codec)
 {
     pj_pool_t *pool;
     pjmedia_codec *codec;
@@ -532,7 +531,7 @@ static pj_status_t silk_alloc_codec(pjmedia_codec_factory *factory,
  * Free codec.
  */
 static pj_status_t silk_dealloc_codec( pjmedia_codec_factory *factory,
-				      pjmedia_codec *codec )
+                                      pjmedia_codec *codec )
 {
     silk_private *silk;
 
@@ -543,7 +542,7 @@ static pj_status_t silk_dealloc_codec( pjmedia_codec_factory *factory,
 
     /* Close codec, if it's not closed. */
     if (silk->enc_ready == PJ_TRUE || silk->dec_ready == PJ_TRUE) {
-    	silk_codec_close(codec);
+        silk_codec_close(codec);
     }
 
     pj_pool_release(silk->pool);
@@ -556,7 +555,7 @@ static pj_status_t silk_dealloc_codec( pjmedia_codec_factory *factory,
  * Init codec.
  */
 static pj_status_t silk_codec_init(pjmedia_codec *codec,
-				   pj_pool_t *pool )
+                                   pj_pool_t *pool )
 {
     PJ_UNUSED_ARG(codec);
     PJ_UNUSED_ARG(pool);
@@ -568,7 +567,7 @@ static pj_status_t silk_codec_init(pjmedia_codec *codec,
  * Open codec.
  */
 static pj_status_t silk_codec_open(pjmedia_codec *codec,
-				   pjmedia_codec_param *attr )
+                                   pjmedia_codec_param *attr )
 {
 
     silk_private *silk;
@@ -584,44 +583,44 @@ static pj_status_t silk_codec_open(pjmedia_codec *codec,
 
     /* Already opened? */
     if (silk->enc_ready || silk->dec_ready)
-	return PJ_SUCCESS;
+        return PJ_SUCCESS;
 
     /* Allocate and initialize encoder */
     err = SKP_Silk_SDK_Get_Encoder_Size(&st_size);
     if (err) {
         PJ_LOG(3,(THIS_FILE, "Failed to get encoder state size (err=%d)",
-		  err));
-	return PJMEDIA_CODEC_EFAILED;
+                  err));
+        return PJMEDIA_CODEC_EFAILED;
     }
     silk->enc_st = pj_pool_zalloc(silk->pool, st_size);
     err = SKP_Silk_SDK_InitEncoder(silk->enc_st, &silk->enc_ctl);
     if (err) {
         PJ_LOG(3,(THIS_FILE, "Failed to init encoder (err=%d)", err));
-	return PJMEDIA_CODEC_EFAILED;
+        return PJMEDIA_CODEC_EFAILED;
     }
 
     /* Check fmtp params */
     enc_use_fec = PJ_TRUE;
     enc_bitrate = sp->bitrate;
     for (i = 0; i < attr->setting.enc_fmtp.cnt; ++i) {
-	pjmedia_codec_fmtp *fmtp = &attr->setting.enc_fmtp;
-	const pj_str_t STR_USEINBANDFEC = {"useinbandfec", 12};
-	const pj_str_t STR_MAXAVERAGEBITRATE = {"maxaveragebitrate", 17};
+        pjmedia_codec_fmtp *fmtp = &attr->setting.enc_fmtp;
+        const pj_str_t STR_USEINBANDFEC = {"useinbandfec", 12};
+        const pj_str_t STR_MAXAVERAGEBITRATE = {"maxaveragebitrate", 17};
 
-	if (!pj_stricmp(&fmtp->param[i].name, &STR_USEINBANDFEC)) {
-	    enc_use_fec = pj_strtoul(&fmtp->param[i].val) != 0;
-	} else if (!pj_stricmp(&fmtp->param[i].name, &STR_MAXAVERAGEBITRATE)) {
-	    enc_bitrate = pj_strtoul(&fmtp->param[i].val);
-	    if (enc_bitrate > sp->max_bitrate) {
-		enc_bitrate = sp->max_bitrate;
-	    }
-	}
+        if (!pj_stricmp(&fmtp->param[i].name, &STR_USEINBANDFEC)) {
+            enc_use_fec = pj_strtoul(&fmtp->param[i].val) != 0;
+        } else if (!pj_stricmp(&fmtp->param[i].name, &STR_MAXAVERAGEBITRATE)) {
+            enc_bitrate = pj_strtoul(&fmtp->param[i].val);
+            if (enc_bitrate > sp->max_bitrate) {
+                enc_bitrate = sp->max_bitrate;
+            }
+        }
     }
 
     /* Setup encoder control for encoding process */
     silk->enc_ready = PJ_TRUE;
     silk->samples_per_frame = FRAME_LENGTH_MS *
-			      attr->info.clock_rate / 1000;
+                              attr->info.clock_rate / 1000;
     silk->pcm_bytes_per_sample = attr->info.pcm_bits_per_sample / 8;
 
     silk->enc_ctl.API_sampleRate        = attr->info.clock_rate;
@@ -642,19 +641,19 @@ static pj_status_t silk_codec_open(pjmedia_codec *codec,
     err = SKP_Silk_SDK_Get_Decoder_Size(&st_size);
     if (err) {
         PJ_LOG(3,(THIS_FILE, "Failed to get decoder state size (err=%d)",
-		  err));
-	return PJMEDIA_CODEC_EFAILED;
+                  err));
+        return PJMEDIA_CODEC_EFAILED;
     }
     silk->dec_st = pj_pool_zalloc(silk->pool, st_size);
     err = SKP_Silk_SDK_InitDecoder(silk->dec_st);
     if (err) {
         PJ_LOG(3,(THIS_FILE, "Failed to init decoder (err=%d)", err));
-	return PJMEDIA_CODEC_EFAILED;
+        return PJMEDIA_CODEC_EFAILED;
     }
 
     /* Setup decoder control for decoding process */
     silk->dec_ctl.API_sampleRate        = attr->info.clock_rate;
-    silk->dec_ctl.framesPerPacket	= 1; /* for proper PLC at start */
+    silk->dec_ctl.framesPerPacket       = 1; /* for proper PLC at start */
     silk->dec_ready = PJ_TRUE;
     silk->dec_buf_sz = attr->info.clock_rate * attr->info.channel_cnt *
                        attr->info.frm_ptime / 1000 *
@@ -664,10 +663,10 @@ static pj_status_t silk_codec_open(pjmedia_codec *codec,
      * SILK packets and split it into individual frames.
      */
     attr->info.max_rx_frame_size = attr->info.max_bps * 
-			           attr->info.frm_ptime / 8 / 1000;
+                                   attr->info.frm_ptime / 8 / 1000;
     if ((attr->info.max_bps * attr->info.frm_ptime) % 8000 != 0)
     {
-	++attr->info.max_rx_frame_size;
+        ++attr->info.max_rx_frame_size;
     }
     attr->info.max_rx_frame_size *= SILK_MAX_FRAMES_PER_PACKET;
 
@@ -694,7 +693,7 @@ static pj_status_t silk_codec_close( pjmedia_codec *codec )
  * Modify codec settings.
  */
 static pj_status_t  silk_codec_modify(pjmedia_codec *codec,
-				      const pjmedia_codec_param *attr )
+                                      const pjmedia_codec_param *attr )
 {
     PJ_TODO(implement_silk_codec_modify);
 
@@ -709,9 +708,9 @@ static pj_status_t  silk_codec_modify(pjmedia_codec *codec,
  * Encode frame.
  */
 static pj_status_t silk_codec_encode(pjmedia_codec *codec,
-				     const struct pjmedia_frame *input,
-				     unsigned output_buf_len,
-				     struct pjmedia_frame *output)
+                                     const struct pjmedia_frame *input,
+                                     unsigned output_buf_len,
+                                     struct pjmedia_frame *output)
 {
     silk_private *silk;
     SKP_int err;
@@ -724,20 +723,20 @@ static pj_status_t silk_codec_encode(pjmedia_codec *codec,
     /* Check frame in size */
     nsamples = input->size >> 1;
     PJ_ASSERT_RETURN(nsamples % silk->samples_per_frame == 0,
-		     PJMEDIA_CODEC_EPCMFRMINLEN);
+                     PJMEDIA_CODEC_EPCMFRMINLEN);
 
     /* Encode */
     output->size = 0;
     out_size = (SKP_int16)output_buf_len;
     err = SKP_Silk_SDK_Encode(silk->enc_st, &silk->enc_ctl,
-			     (SKP_int16*)input->buf, nsamples,
-			     (SKP_uint8*)output->buf, &out_size);
+                             (SKP_int16*)input->buf, nsamples,
+                             (SKP_uint8*)output->buf, &out_size);
     if (err) {
-	PJ_LOG(3, (THIS_FILE, "Failed to encode frame (err=%d)", err));
-	output->type = PJMEDIA_FRAME_TYPE_NONE;
-	if (err == SKP_SILK_ENC_PAYLOAD_BUF_TOO_SHORT)
-	    return PJMEDIA_CODEC_EFRMTOOSHORT;
-	return PJMEDIA_CODEC_EFAILED;
+        PJ_LOG(3, (THIS_FILE, "Failed to encode frame (err=%d)", err));
+        output->type = PJMEDIA_FRAME_TYPE_NONE;
+        if (err == SKP_SILK_ENC_PAYLOAD_BUF_TOO_SHORT)
+            return PJMEDIA_CODEC_EFRMTOOSHORT;
+        return PJMEDIA_CODEC_EFAILED;
     }
 
     output->size = out_size;
@@ -752,11 +751,11 @@ static pj_status_t silk_codec_encode(pjmedia_codec *codec,
  * Get frames in the packet.
  */
 static pj_status_t  silk_codec_parse( pjmedia_codec *codec,
-				     void *pkt,
-				     pj_size_t pkt_size,
-				     const pj_timestamp *ts,
-				     unsigned *frame_cnt,
-				     pjmedia_frame frames[])
+                                     void *pkt,
+                                     pj_size_t pkt_size,
+                                     const pj_timestamp *ts,
+                                     unsigned *frame_cnt,
+                                     pjmedia_frame frames[])
 {
     silk_private *silk;
     SKP_Silk_TOC_struct toc;
@@ -772,7 +771,7 @@ static pj_status_t  silk_codec_parse( pjmedia_codec *codec,
     for (i = 0; i < count; i++) {
         frames[i].type = PJMEDIA_FRAME_TYPE_AUDIO;
         frames[i].bit_info = (((unsigned)ts->u64 & 0xFFFF) << 16) |
-                              (((unsigned)pkt & 0xFF) << 8) |
+                              (((pj_size_t)pkt & 0xFF) << 8) |
                               (toc.framesInPacket << 4) | i;
         frames[i].buf = pkt;
         frames[i].size = pkt_size;
@@ -784,9 +783,9 @@ static pj_status_t  silk_codec_parse( pjmedia_codec *codec,
 }
 
 static pj_status_t silk_codec_decode(pjmedia_codec *codec,
-				     const struct pjmedia_frame *input,
-				     unsigned output_buf_len,
-				     struct pjmedia_frame *output)
+                                     const struct pjmedia_frame *input,
+                                     unsigned output_buf_len,
+                                     struct pjmedia_frame *output)
 {
     silk_private *silk;
     SKP_int16 out_size;
@@ -846,11 +845,11 @@ static pj_status_t silk_codec_decode(pjmedia_codec *codec,
 
             *size = nsamples;
             err = SKP_Silk_SDK_Decode(silk->dec_st, &silk->dec_ctl,
-			              0, /* Normal frame flag */
-			              input->buf, input->size,
-			              buf, size);
+                                      0, /* Normal frame flag */
+                                      input->buf, input->size,
+                                      buf, size);
             if (err) {
-	        PJ_LOG(3, (THIS_FILE, "Failed to decode frame (err=%d)",
+                PJ_LOG(3, (THIS_FILE, "Failed to decode frame (err=%d)",
                                       err));
                 *size = 0;
             } else {
@@ -911,8 +910,8 @@ static pj_status_t silk_codec_decode(pjmedia_codec *codec,
  * Recover lost frame.
  */
 static pj_status_t  silk_codec_recover(pjmedia_codec *codec,
-				       unsigned output_buf_len,
-				       struct pjmedia_frame *output)
+                                       unsigned output_buf_len,
+                                       struct pjmedia_frame *output)
 {
     silk_private *silk;
     SKP_int16 out_size;
@@ -923,17 +922,17 @@ static pj_status_t  silk_codec_recover(pjmedia_codec *codec,
 
     out_size = (SKP_int16)output_buf_len / silk->pcm_bytes_per_sample;
     err = SKP_Silk_SDK_Decode(silk->dec_st, &silk->dec_ctl,
-			      1, /* Lost frame flag */
-			      NULL,
-			      0,
-			      output->buf,
-			      &out_size);
+                              1, /* Lost frame flag */
+                              NULL,
+                              0,
+                              output->buf,
+                              &out_size);
     if (err) {
-	PJ_LOG(3, (THIS_FILE, "Failed to conceal lost frame (err=%d)", err));
-	output->type = PJMEDIA_FRAME_TYPE_NONE;
-	output->buf = NULL;
-	output->size = 0;
-	return PJMEDIA_CODEC_EFAILED;
+        PJ_LOG(3, (THIS_FILE, "Failed to conceal lost frame (err=%d)", err));
+        output->type = PJMEDIA_FRAME_TYPE_NONE;
+        output->buf = NULL;
+        output->size = 0;
+        return PJMEDIA_CODEC_EFAILED;
     }
 
     output->size = out_size * silk->pcm_bytes_per_sample;
