@@ -1,4 +1,3 @@
-/* $Id: json.cpp 4761 2014-02-24 09:02:44Z nanang $ */
 /*
  * Copyright (C) 2013 Teluu Inc. (http://www.teluu.com)
  *
@@ -21,55 +20,55 @@
 #include <pj/file_io.h>
 #include "util.hpp"
 
-#define THIS_FILE	"json.cpp"
+#define THIS_FILE       "json.cpp"
 
 using namespace pj;
 using namespace std;
 
 /* Json node operations */
-static bool		jsonNode_hasUnread(const ContainerNode*);
-static string		jsonNode_unreadName(const ContainerNode*n)
-				            throw(Error);
-static float		jsonNode_readNumber(const ContainerNode*,
-            		                    const string&)
-					      throw(Error);
-static bool		jsonNode_readBool(const ContainerNode*,
-           		                  const string&)
-					  throw(Error);
-static string		jsonNode_readString(const ContainerNode*,
-             		                    const string&)
-					    throw(Error);
-static StringVector	jsonNode_readStringVector(const ContainerNode*,
-                   	                          const string&)
-						  throw(Error);
-static ContainerNode	jsonNode_readContainer(const ContainerNode*,
-                    	                       const string &)
-					       throw(Error);
-static ContainerNode	jsonNode_readArray(const ContainerNode*,
-                    	                   const string &)
-					   throw(Error);
-static void		jsonNode_writeNumber(ContainerNode*,
-           		                     const string &name,
-           		                     float num)
-           		                     throw(Error);
-static void		jsonNode_writeBool(ContainerNode*,
-           		                   const string &name,
-           		                   bool value)
-					   throw(Error);
-static void		jsonNode_writeString(ContainerNode*,
-           		                     const string &name,
-           		                     const string &value)
-					     throw(Error);
-static void		jsonNode_writeStringVector(ContainerNode*,
-           		                           const string &name,
-           		                           const StringVector &value)
-					           throw(Error);
-static ContainerNode 	jsonNode_writeNewContainer(ContainerNode*,
-                     	                           const string &name)
-					           throw(Error);
-static ContainerNode 	jsonNode_writeNewArray(ContainerNode*,
-                     	                       const string &name)
-					       throw(Error);
+static bool             jsonNode_hasUnread(const ContainerNode*);
+static string           jsonNode_unreadName(const ContainerNode*n)
+                                            PJSUA2_THROW(Error);
+static float            jsonNode_readNumber(const ContainerNode*,
+                                            const string&)
+                                              PJSUA2_THROW(Error);
+static bool             jsonNode_readBool(const ContainerNode*,
+                                          const string&)
+                                          PJSUA2_THROW(Error);
+static string           jsonNode_readString(const ContainerNode*,
+                                            const string&)
+                                            PJSUA2_THROW(Error);
+static StringVector     jsonNode_readStringVector(const ContainerNode*,
+                                                  const string&)
+                                                  PJSUA2_THROW(Error);
+static ContainerNode    jsonNode_readContainer(const ContainerNode*,
+                                               const string &)
+                                               PJSUA2_THROW(Error);
+static ContainerNode    jsonNode_readArray(const ContainerNode*,
+                                           const string &)
+                                           PJSUA2_THROW(Error);
+static void             jsonNode_writeNumber(ContainerNode*,
+                                             const string &name,
+                                             float num)
+                                             PJSUA2_THROW(Error);
+static void             jsonNode_writeBool(ContainerNode*,
+                                           const string &name,
+                                           bool value)
+                                           PJSUA2_THROW(Error);
+static void             jsonNode_writeString(ContainerNode*,
+                                             const string &name,
+                                             const string &value)
+                                             PJSUA2_THROW(Error);
+static void             jsonNode_writeStringVector(ContainerNode*,
+                                                   const string &name,
+                                                   const StringVector &value)
+                                                   PJSUA2_THROW(Error);
+static ContainerNode    jsonNode_writeNewContainer(ContainerNode*,
+                                                   const string &name)
+                                                   PJSUA2_THROW(Error);
+static ContainerNode    jsonNode_writeNewArray(ContainerNode*,
+                                               const string &name)
+                                               PJSUA2_THROW(Error);
 
 static container_node_op json_op =
 {
@@ -96,13 +95,13 @@ JsonDocument::JsonDocument()
     pj_caching_pool_init(&cp, NULL, 0);
     pool = pj_pool_create(&cp.factory, "jsondoc", 512, 512, NULL);
     if (!pool)
-	PJSUA2_RAISE_ERROR(PJ_ENOMEM);
+        PJSUA2_RAISE_ERROR(PJ_ENOMEM);
 }
 
 JsonDocument::~JsonDocument()
 {
     if (pool)
-	pj_pool_release(pool);
+        pj_pool_release(pool);
     pj_caching_pool_destroy(&cp);
 }
 
@@ -114,16 +113,18 @@ void JsonDocument::initRoot() const
     rootNode.data.data2 = root->value.children.next;
 }
 
-void JsonDocument::loadFile(const string &filename) throw(Error)
+void JsonDocument::loadFile(const string &filename) PJSUA2_THROW(Error)
 {
     if (root)
-	PJSUA2_RAISE_ERROR3(PJ_EINVALIDOP, "JsonDocument.loadString()",
-	                    "Document already initialized");
+        PJSUA2_RAISE_ERROR3(PJ_EINVALIDOP, "JsonDocument.loadString()",
+                            "Document already initialized");
 
     if (!pj_file_exists(filename.c_str()))
-	PJSUA2_RAISE_ERROR(PJ_ENOTFOUND);
+        PJSUA2_RAISE_ERROR(PJ_ENOTFOUND);
 
     pj_ssize_t size = (pj_ssize_t)pj_file_size(filename.c_str());
+    if (size <= 0)
+        PJSUA2_RAISE_ERROR(PJ_ETOOSMALL);
     pj_status_t status;
 
     char *buffer = (char*)pj_pool_alloc(pool, size+1);
@@ -136,30 +137,30 @@ void JsonDocument::loadFile(const string &filename) throw(Error)
 
     status = pj_file_open(pool, filename.c_str(), PJ_O_RDONLY, &fd);
     if (status != PJ_SUCCESS)
-	goto on_error;
+        goto on_error;
 
     status = pj_file_read(fd, buffer, &size);
     if (status != PJ_SUCCESS)
-	goto on_error;
+        goto on_error;
 
     pj_file_close(fd);
     fd = NULL;
 
     if (size <= 0) {
-	status = PJ_EEOF;
-	goto on_error;
+        status = PJ_EEOF;
+        goto on_error;
     }
 
     parse_size = (unsigned)size;
     root = pj_json_parse(pool, buffer, &parse_size, &err_info);
     if (root == NULL) {
-	pj_ansi_snprintf(err_msg, sizeof(err_msg),
-	                 "JSON parsing failed: syntax error in file '%s' at "
-	                 "line %d column %d",
-	                 filename.c_str(), err_info.line, err_info.col);
-	PJ_LOG(1,(THIS_FILE, err_msg));
-	status = PJLIB_UTIL_EINJSON;
-	goto on_error;
+        pj_ansi_snprintf(err_msg, sizeof(err_msg),
+                         "JSON parsing failed: syntax error in file '%s' at "
+                         "line %d column %d",
+                         filename.c_str(), err_info.line, err_info.col);
+        PJ_LOG(1,(THIS_FILE, "%s", err_msg));
+        status = PJLIB_UTIL_EINJSON;
+        goto on_error;
     }
 
     initRoot();
@@ -167,18 +168,18 @@ void JsonDocument::loadFile(const string &filename) throw(Error)
 
 on_error:
     if (fd)
-	pj_file_close(fd);
+        pj_file_close(fd);
     if (err_msg[0])
-	PJSUA2_RAISE_ERROR3(status, "loadFile()", err_msg);
+        PJSUA2_RAISE_ERROR3(status, "loadFile()", err_msg);
     else
-	PJSUA2_RAISE_ERROR(status);
+        PJSUA2_RAISE_ERROR(status);
 }
 
-void JsonDocument::loadString(const string &input) throw(Error)
+void JsonDocument::loadString(const string &input) PJSUA2_THROW(Error)
 {
     if (root)
-	PJSUA2_RAISE_ERROR3(PJ_EINVALIDOP, "JsonDocument.loadString()",
-	                    "Document already initialized");
+        PJSUA2_RAISE_ERROR3(PJ_EINVALIDOP, "JsonDocument.loadString()",
+                            "Document already initialized");
 
     unsigned size = (unsigned)input.size();
     char *buffer = (char*)pj_pool_alloc(pool, size+1);
@@ -189,13 +190,13 @@ void JsonDocument::loadString(const string &input) throw(Error)
 
     root = pj_json_parse(pool, buffer, &parse_size, &err_info);
     if (root == NULL) {
-	char err_msg[80];
+        char err_msg[80];
 
-	pj_ansi_snprintf(err_msg, sizeof(err_msg),
-	                 "JSON parsing failed at line %d column %d",
-	                 err_info.line, err_info.col);
-	PJ_LOG(1,(THIS_FILE, err_msg));
-	PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, "loadString()", err_msg);
+        pj_ansi_snprintf(err_msg, sizeof(err_msg),
+                         "JSON parsing failed at line %d column %d",
+                         err_info.line, err_info.col);
+        PJ_LOG(1,(THIS_FILE, "%s", err_msg));
+        PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, "loadString()", err_msg);
     }
     initRoot();
 }
@@ -206,15 +207,15 @@ struct save_file_data
 };
 
 static pj_status_t json_file_writer(const char *s,
-				    unsigned size,
-				    void *user_data)
+                                    unsigned size,
+                                    void *user_data)
 {
     save_file_data *sd = (save_file_data*)user_data;
     pj_ssize_t ssize = (pj_ssize_t)size;
     return pj_file_write(sd->fd, s, &ssize);
 }
 
-void JsonDocument::saveFile(const string &filename) throw(Error)
+void JsonDocument::saveFile(const string &filename) PJSUA2_THROW(Error)
 {
     struct save_file_data sd;
     pj_status_t status;
@@ -224,18 +225,18 @@ void JsonDocument::saveFile(const string &filename) throw(Error)
 
     status = pj_file_open(pool, filename.c_str(), PJ_O_WRONLY, &sd.fd);
     if (status != PJ_SUCCESS)
-	PJSUA2_RAISE_ERROR(status);
+        PJSUA2_RAISE_ERROR(status);
 
     status = pj_json_writef(root, &json_file_writer, &sd.fd);
     pj_file_close(sd.fd);
 
     if (status != PJ_SUCCESS)
-	PJSUA2_RAISE_ERROR(status);
+        PJSUA2_RAISE_ERROR(status);
 }
 
 struct save_string_data
 {
-    string	output;
+    string      output;
 };
 
 static pj_status_t json_string_writer(const char *s,
@@ -247,7 +248,7 @@ static pj_status_t json_string_writer(const char *s,
     return PJ_SUCCESS;
 }
 
-string JsonDocument::saveString() throw(Error)
+string JsonDocument::saveString() PJSUA2_THROW(Error)
 {
     struct save_string_data sd;
     pj_status_t status;
@@ -257,7 +258,7 @@ string JsonDocument::saveString() throw(Error)
 
     status = pj_json_writef(root, &json_string_writer, &sd);
     if (status != PJ_SUCCESS)
-	PJSUA2_RAISE_ERROR(status);
+        PJSUA2_RAISE_ERROR(status);
 
     return sd.output;
 }
@@ -265,9 +266,9 @@ string JsonDocument::saveString() throw(Error)
 ContainerNode & JsonDocument::getRootContainer() const
 {
     if (!root) {
-	root = allocElement();
-	pj_json_elem_obj(root, NULL);
-	initRoot();
+        root = allocElement();
+        pj_json_elem_obj(root, NULL);
+        initRoot();
     }
 
     return rootNode;
@@ -286,9 +287,9 @@ pj_pool_t *JsonDocument::getPool()
 ///////////////////////////////////////////////////////////////////////////////
 struct json_node_data
 {
-    JsonDocument	*doc;
-    pj_json_elem	*jnode;
-    pj_json_elem	*childPtr;
+    JsonDocument        *doc;
+    pj_json_elem        *jnode;
+    pj_json_elem        *childPtr;
 };
 
 static bool jsonNode_hasUnread(const ContainerNode *node)
@@ -303,38 +304,37 @@ static void json_verify(struct json_node_data *jdat,
                         pj_json_val_type type)
 {
     if (jdat->childPtr == (pj_json_elem*)&jdat->jnode->value.children)
-	PJSUA2_RAISE_ERROR3(PJ_EEOF, op, "No unread element");
+        PJSUA2_RAISE_ERROR3(PJ_EEOF, op, "No unread element");
 
     /* If name is specified, then check if the names match, except
      * when the node name itself is empty and the parent node is
      * an array, then ignore the checking (JSON doesn't allow array
      * elements to have name).
      */
-    if (name.size() && name.compare(0, name.size(),
-                                    jdat->childPtr->name.ptr,
-                                    jdat->childPtr->name.slen) &&
-        jdat->childPtr->name.slen &&
-        jdat->jnode->type != PJ_JSON_VAL_ARRAY)
+    if (jdat->jnode->type != PJ_JSON_VAL_ARRAY &&
+        name.size() && jdat->childPtr->name.slen &&
+        name.compare(0, name.size(), jdat->childPtr->name.ptr,
+                                     jdat->childPtr->name.slen))
     {
-	char err_msg[80];
-	pj_ansi_snprintf(err_msg, sizeof(err_msg),
-	                 "Name mismatch: expecting '%s' got '%.*s'",
-	                 name.c_str(), (int)jdat->childPtr->name.slen,
-	                 jdat->childPtr->name.ptr);
-	PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, op, err_msg);
+        char err_msg[80];
+        pj_ansi_snprintf(err_msg, sizeof(err_msg),
+                         "Name mismatch: expecting '%s' got '%.*s'",
+                         name.c_str(), (int)jdat->childPtr->name.slen,
+                         jdat->childPtr->name.ptr);
+        PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, op, err_msg);
     }
 
     if (type != PJ_JSON_VAL_NULL && jdat->childPtr->type != type) {
-	char err_msg[80];
-	pj_ansi_snprintf(err_msg, sizeof(err_msg),
-	                 "Type mismatch: expecting %d got %d",
-	                 type, jdat->childPtr->type);
-	PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, op, err_msg);
+        char err_msg[80];
+        pj_ansi_snprintf(err_msg, sizeof(err_msg),
+                         "Type mismatch: expecting %d got %d",
+                         type, jdat->childPtr->type);
+        PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, op, err_msg);
     }
 }
 
 static string jsonNode_unreadName(const ContainerNode *node)
-				  throw(Error)
+                                  PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "unreadName()", "", PJ_JSON_VAL_NULL);
@@ -342,8 +342,8 @@ static string jsonNode_unreadName(const ContainerNode *node)
 }
 
 static float jsonNode_readNumber(const ContainerNode *node,
-            		         const string &name)
-				 throw(Error)
+                                 const string &name)
+                                 PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readNumber()", name, PJ_JSON_VAL_NUMBER);
@@ -352,8 +352,8 @@ static float jsonNode_readNumber(const ContainerNode *node,
 }
 
 static bool jsonNode_readBool(const ContainerNode *node,
-			      const string &name)
-			      throw(Error)
+                              const string &name)
+                              PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readBool()", name, PJ_JSON_VAL_BOOL);
@@ -362,8 +362,8 @@ static bool jsonNode_readBool(const ContainerNode *node,
 }
 
 static string jsonNode_readString(const ContainerNode *node,
-				  const string &name)
-				  throw(Error)
+                                  const string &name)
+                                  PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readString()", name, PJ_JSON_VAL_STRING);
@@ -372,8 +372,8 @@ static string jsonNode_readString(const ContainerNode *node,
 }
 
 static StringVector jsonNode_readStringVector(const ContainerNode *node,
-                   	                      const string &name)
-					      throw(Error)
+                                              const string &name)
+                                              PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readStringVector()", name, PJ_JSON_VAL_ARRAY);
@@ -381,16 +381,16 @@ static StringVector jsonNode_readStringVector(const ContainerNode *node,
     StringVector result;
     pj_json_elem *child = jdat->childPtr->value.children.next;
     while (child != (pj_json_elem*)&jdat->childPtr->value.children) {
-	if (child->type != PJ_JSON_VAL_STRING) {
-	    char err_msg[80];
-	    pj_ansi_snprintf(err_msg, sizeof(err_msg),
-			     "Elements not string but type %d",
-			     child->type);
-	    PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, "readStringVector()",
-	                        err_msg);
-	}
-	result.push_back(pj2Str(child->value.str));
-	child = child->next;
+        if (child->type != PJ_JSON_VAL_STRING) {
+            char err_msg[80];
+            pj_ansi_snprintf(err_msg, sizeof(err_msg),
+                             "Elements not string but type %d",
+                             child->type);
+            PJSUA2_RAISE_ERROR3(PJLIB_UTIL_EINJSON, "readStringVector()",
+                                err_msg);
+        }
+        result.push_back(pj2Str(child->value.str));
+        child = child->next;
     }
 
     jdat->childPtr = jdat->childPtr->next;
@@ -398,8 +398,8 @@ static StringVector jsonNode_readStringVector(const ContainerNode *node,
 }
 
 static ContainerNode jsonNode_readContainer(const ContainerNode *node,
-                    	                    const string &name)
-					    throw(Error)
+                                            const string &name)
+                                            PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readContainer()", name, PJ_JSON_VAL_OBJ);
@@ -416,8 +416,8 @@ static ContainerNode jsonNode_readContainer(const ContainerNode *node,
 }
 
 static ContainerNode jsonNode_readArray(const ContainerNode *node,
-                    	                const string &name)
-					throw(Error)
+                                        const string &name)
+                                        PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     json_verify(jdat, "readArray()", name, PJ_JSON_VAL_ARRAY);
@@ -441,9 +441,9 @@ static pj_str_t alloc_name(JsonDocument *doc, const string &name)
 }
 
 static void jsonNode_writeNumber(ContainerNode *node,
-           		         const string &name,
-           		         float num)
-           		         throw(Error)
+                                 const string &name,
+                                 float num)
+                                 PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
@@ -453,9 +453,9 @@ static void jsonNode_writeNumber(ContainerNode *node,
 }
 
 static void jsonNode_writeBool(ContainerNode *node,
-           		       const string &name,
-           		       bool value)
-			       throw(Error)
+                               const string &name,
+                               bool value)
+                               PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
@@ -465,9 +465,9 @@ static void jsonNode_writeBool(ContainerNode *node,
 }
 
 static void jsonNode_writeString(ContainerNode *node,
-           		         const string &name,
-           		         const string &value)
-				 throw(Error)
+                                 const string &name,
+                                 const string &value)
+                                 PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
@@ -480,9 +480,9 @@ static void jsonNode_writeString(ContainerNode *node,
 }
 
 static void jsonNode_writeStringVector(ContainerNode *node,
-           		               const string &name,
-           		               const StringVector &value)
-				       throw(Error)
+                                       const string &name,
+                                       const StringVector &value)
+                                       PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
@@ -490,20 +490,20 @@ static void jsonNode_writeStringVector(ContainerNode *node,
 
     pj_json_elem_array(el, &nm);
     for (unsigned i=0; i<value.size(); ++i) {
-	pj_str_t new_val;
+        pj_str_t new_val;
 
-	pj_strdup2(jdat->doc->getPool(), &new_val, value[i].c_str());
-	pj_json_elem *child = jdat->doc->allocElement();
-	pj_json_elem_string(child, NULL, &new_val);
-	pj_json_elem_add(el, child);
+        pj_strdup2(jdat->doc->getPool(), &new_val, value[i].c_str());
+        pj_json_elem *child = jdat->doc->allocElement();
+        pj_json_elem_string(child, NULL, &new_val);
+        pj_json_elem_add(el, child);
     }
 
     pj_json_elem_add(jdat->jnode, el);
 }
 
 static ContainerNode jsonNode_writeNewContainer(ContainerNode *node,
-                     	                        const string &name)
-					        throw(Error)
+                                                const string &name)
+                                                PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
@@ -523,8 +523,8 @@ static ContainerNode jsonNode_writeNewContainer(ContainerNode *node,
 }
 
 static ContainerNode jsonNode_writeNewArray(ContainerNode *node,
-                     	                    const string &name)
-					    throw(Error)
+                                            const string &name)
+                                            PJSUA2_THROW(Error)
 {
     json_node_data *jdat = (json_node_data*)&node->data;
     pj_json_elem *el = jdat->doc->allocElement();
